extern "C" {
#include "mathlink.h"
}
#include "simpleString.hpp"
#pragma warning(disable:4786)
#include "Choice.hpp"
#ifdef HAS_INCLUDE_NO_DOTS
#include <vector>
#else
#include <vector.h>
#endif
#ifdef HAS_INCLUDE_NO_DOTS
#include <functional>
#else
#include <function.h>
#endif
#include <algorithm>
#include "RecordHere.hpp"

#include "vcpp.hpp"

#define DBG() { run_time_error(__FILE__,__LINE__);}

void run_time_error(const char * s,int L) {
  cout << "Error in file " << s << "one line " << L << '\n';
  cout.flush();
};

struct function_type {};
struct string_type {};
struct symbol_type {};
struct integer_type {};

class Composition {
public:
  simpleString d_s;
  int d_i;
  Composition * d_v;
  int d_type;
  bool d_valid;
  int d_sz;
  static const int SYMBOL;
  static const int STRING;
  static const int FUNCTION;
  static const int INTEGER;
  Composition() : 
       d_s(), d_i(0), d_v((Composition *)0), d_valid(false), d_sz(0) {};
  Composition(const char * s,Composition * p,int sz,function_type) : d_s(s), 
       d_v(p), d_type(FUNCTION),d_valid(true), d_sz(sz) {};
  Composition(const char * s,string_type) : 
     d_s(s), d_v((Composition *)0), d_type(STRING), d_valid(true), d_sz(0) {};
  Composition(const char * s,symbol_type) : 
     d_s(s), d_v((Composition *)0), d_type(SYMBOL), d_valid(true),d_sz(0) {};
  Composition(int i,integer_type) : d_i(i), d_v((Composition *) 0), d_type(INTEGER), d_valid(true),d_sz(0) {};
  Composition(Composition & x) : 
      d_s(x.d_s), d_i(x.d_i), d_v((Composition *)0), 
      d_type(x.d_type), d_valid(x.d_valid), d_sz(x.d_sz) {
    operator=(x);
  };
  void operator=(const Composition & x) { 
    if(this!=&x) {
      d_s=x.d_s; 
      d_i=x.d_i; 
      RECORDHERE(delete [] d_v;)
      d_sz = x.d_sz;
      RECORDHERE(d_v=new Composition[d_sz]; )
      Composition * p =  x.d_v;
      Composition * q =  d_v;
      for(int i=1;i<=x.d_sz;++i,++p,++q) { RECORDHERE(delete q);q = p;};
      d_type=x.d_type;
      d_valid=x.d_valid;
      d_sz=x.d_sz; 
    };
  };
  ~Composition() {
    RECORDHERE(delete [] d_v;)
  };
  bool valid() const { return d_valid;};
  void put(MLINK mlp) const {
    if(!valid()) DBG();
    if(d_type==FUNCTION) {
      const int sz = d_sz;
      MLPutFunction(mlp,d_s.chars(),sz);
      Composition * w = d_v;
      for(int i=1;i<=d_sz;++i,++w) {
        w->put(mlp);
      };
    } else if(d_type==SYMBOL) {
      MLPutSymbol(mlp,d_s.chars());
    } else if(d_type==STRING) {
      MLPutString(mlp,d_s.chars());
    } else if(d_type==INTEGER) {
      MLPutInteger(mlp,d_i);
    } else DBG();
  };
  void print(MyOstream & os) const {
    if(!valid()) DBG();
    if(d_type==FUNCTION) {
      os << d_s.chars() << '[';
      Composition * w = d_v;
      for(int i=1;i<=d_sz;++i,++w) {
        if(i!=1) os << ",";
        w->print(os);
      };
    } else if(d_type==SYMBOL) {
      os << d_s.chars();
    } else if(d_type==STRING) {
      os << d_s.chars();
    } else if(d_type==INTEGER) {
      os << d_i;
    } else DBG();
  };
};

Composition GrabAnExpression(MLINK mlp) {
  Composition result;
  char * s;
  int type = MLGetType(mlp);
  if(type==MLTKFUNC) {
cerr << "FUNCTION\n";
      function_type TYPE;
      long L;
      MLGetFunction(mlp,(const char **)&s,&L);
      RECORDHERE(Composition *  v = new Composition[L];)
      Composition *  w = v;
      for(int i=1;i<=L;++i,++w) {
        *w = GrabAnExpression(mlp);
      };
      result = Composition((const char *)s,v,(int)L,TYPE);
      MLDisownSymbol(mlp,s);
    } else if(type==Composition::SYMBOL) {
cerr << "SYMBOL\n";
      symbol_type TYPE;
      MLGetSymbol(mlp,(const char **)&s);
      result = Composition(s,TYPE);
      MLDisownSymbol(mlp,s);
    } else if(type==Composition::STRING) {
cerr << "STRING\n";
      string_type TYPE;
      MLGetString(mlp,(const char **)&s);
      result = Composition(s,TYPE);
      MLDisownString(mlp,s);
    } else if(type==Composition::INTEGER) {
cerr << "INTEGER\n";
      integer_type TYPE;
      int i;
      MLGetInteger(mlp,&i);
      result = Composition(i,TYPE);
    } else DBG();
    return result;
};

void OutputAVector(MLINK mlp,const vector<simpleString> & x) {
  MLPutFunction(mlp,"List",x.size());
  vector<simpleString>::const_iterator w = x.begin();
  vector<simpleString>::const_iterator e = x.end();
  while(w!=e) {
    MLPutString(mlp,(*w).chars());
    ++w;
  };
};

vector<simpleString> d_usedVariables;
vector<simpleString> d_localVariables;
vector<simpleString> d_globalVariables;

int _getAndPrintComposition() {
  Composition comp(GrabAnExpression(stdlink));
  comp.print(cout);
  MLNewPacket(stdlink);
  comp.put(stdlink);
  return 1;
};
 
int _getUsedVariables() {
  MLNewPacket(stdlink);
  OutputAVector(stdlink,d_usedVariables);
  return 1;
};

int _getLocalVariables() {
  MLNewPacket(stdlink);
  OutputAVector(stdlink,d_localVariables);
  return 1;
};

int _getGlobalVariables() {
  MLNewPacket(stdlink);
  OutputAVector(stdlink,d_localVariables);
  return 1;
};
 
void GrabEntriesFromList(const Composition & comp) {
  if(comp.d_type!=Composition::FUNCTION) DBG();
  DBG();
};

void IdentifyLocalVariables(const Composition & comp) {
  d_localVariables.erase(d_localVariables.begin(),d_localVariables.end());
  if(comp.d_type==Composition::SYMBOL)  {
    // do nothing
  } else if(comp.d_type==Composition::STRING)  {
    // do nothing
  } else if(comp.d_type==Composition::INTEGER)  {
    // do nothing
  } else if(comp.d_type==Composition::FUNCTION) {
    if(comp.d_s=="Module") {
      GrabEntriesFromList(*comp.d_v);
    } else if(comp.d_s=="RuleDelayed") {
      Composition * w = comp.d_v;
      ++w;
      GrabEntriesFromList(*w);
    } else DBG();
  } else DBG();
};

void IdentifyGlobalUsedVariables(const Composition & comp) {
  d_globalVariables.erase(d_globalVariables.begin(),d_globalVariables.end());
  d_usedVariables.erase(d_usedVariables.begin(),d_usedVariables.end());
  if(comp.d_type==Composition::SYMBOL)  {
    vector<simpleString>::const_iterator w = d_localVariables.begin();
    vector<simpleString>::const_iterator e = d_localVariables.end();
    if(find(w,e,comp.d_s)==e) d_globalVariables.push_back(comp.d_s);
    d_usedVariables.push_back(comp.d_s);
  } else if(comp.d_type==Composition::STRING)  {
    // do nothing
  } else if(comp.d_type==Composition::INTEGER)  {
    // do nothing
  } else if(comp.d_type==Composition::FUNCTION) {
    Composition * w = comp.d_v;
    Composition * e = (comp.d_v+comp.d_sz);
    while(w!=e) {
      IdentifyGlobalUsedVariables(*w);
      ++w;
    };
  } else DBG();
};

int _Holdify() {
  Composition comp(GrabAnExpression(stdlink));
  IdentifyLocalVariables(comp);
  IdentifyGlobalUsedVariables(comp);
  MLNewPacket(stdlink);
  MLPutSymbol(stdlink,"Null");
  return 1;
};

const int Composition::SYMBOL = 1;;
const int Composition::STRING = 2;
const int Composition::FUNCTION = 3;
const int Composition::INTEGER = 4;

int main(int argc,char ** argv) {
  return MLMain(argc,argv);
};          
