(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 4.0,
MathReader 4.0, or any compatible application. The data for the notebook 
starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[      9552,        271]*)
(*NotebookOutlinePosition[     10383,        300]*)
(*  CellTagsIndexPosition[     10339,        296]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\

Example of the LDU Decomposition: 
The Bounded Real Lemma
\
\>", "Title",
  TextAlignment->Center],

Cell[CellGroupData[{

Cell["Bounded Real Lemma", "Section"],

Cell[TextData[{
  "\nLet the stable system be given by\n\n",
  Cell[BoxData[
      RowBox[{
        RowBox[{
          RowBox[{
            RowBox[{
              RowBox[{
                RowBox[{
                  RowBox[{
                    RowBox[{
                      RowBox[{
                        RowBox[{"\t\t", 
                          RowBox[{
                            StyleBox["H",
                              FontSize->16], 
                            StyleBox[" ",
                              FontSize->16], 
                            StyleBox[":",
                              FontSize->16], 
                            RowBox[{
                              RowBox[{
                                StyleBox["(",
                                  FontSize->16], 
                                StyleBox["\[NegativeThinSpace]",
                                  FontSize->16], 
                                RowBox[{
                                  StyleBox[GridBox[{
                                        {\(dx\/dt\)},
                                        {"z"}
                                        }],
                                    FontSize->16], 
                                  
                                  StyleBox[
                                    "\[NegativeThinSpace]  \
\[NegativeThinSpace]",
                                    FontSize->16], 
                                  StyleBox[GridBox[{
                                        {"=", \(A\ \ x\  + \ B\ w\)},
                                        {"=", \(C\_1\ x\  + \ D\_1\ w\)}
                                        }],
                                    FontSize->16], 
                                  StyleBox["\[NegativeThinSpace]",
                                    FontSize->16], 
                                  StyleBox["\n",
                                    FontSize->16], 
                                  StyleBox["\n",
                                    FontSize->16], 
                                  StyleBox[" ",
                                    FontSize->16], 
                                  StyleBox["The",
                                    FontSize->16], 
                                  StyleBox[" ",
                                    FontSize->16], 
                                  SubscriptBox[
                                    StyleBox["\[ScriptCapitalH]",
                                      FontSize->16], "\[Infinity]"], " ", 
                                  "norm", " ", "of", " ", "the", " ", 
                                  "transfer", " ", "function", "  ", "H", 
                                  " ", "is", " ", "\n", "bounded", " ", "by", 
                                  "  ", "\[Gamma]", "  ", "if", " ", "and", 
                                  " ", "only", " ", "if", " ", "the", " ", 
                                  "following", " ", "statements", " ", 
                                  \(holds : \n\t\t\t\t\n
                                    \t\t\t\t\ \ \ \ \ \ \ \ \ I\)}], ")"}], 
                              "  ", "||", "H", \( || \_\[Infinity]\), " ", 
                              \(\( < \ \ \[Gamma]\)\n\n\n\t\t\t\tII\)}]}], 
                          ")"}], "     ", \(\[Gamma]\^2\)}], " ", "-", " ", 
                      \(\(D\_1\) \(D\_1\)\&\(\ \ T\)\)}], "   ", ">", "  ", 
                    "0"}], ",", " ", "and"}], "\n", 
                "                            ", 
                \(X\ \ A\  + \ A\^T\ X\ \  + \ X\ B\ B\^T\ X\  +  ... \), 
                "\n", "                           ", 
                \(\((X\ \ \ B\ \ \ \(D\_1\)\&\(\ \ T\)\  + \ 
                        \(C\_1\)\&\(\ \ T\)\ )\)\ 
                    \((\ \ \[Gamma]\^2\  - \ \(D\_1\) \(D\_1\)\&\(\ \ T\))
                        \)\^\(-1\)\ 
                    \((X\ \ \ B\ \ \ \(D\_1\)\&\(\ \ T\)\  + \ 
                          \(C\_1\)\&\(\ \ T\)\ )\)\^T\  < \ 0\), "\n", "\t", 
                "\n", "\n", "\t           ", "III"}], ")"}], "   ", 
            RowBox[{"(", "\[NegativeThinSpace]", GridBox[{
                  {\(X\ \ A + A\^T\ \ X\), \(X\ \ B\), \(\ 
                    \(C\_1\)\&\(\ \ T\)\)},
                  {\(B\^T\ \ X\), \(-I\), \(\(D\_1\)\&\(\ \ T\)\)},
                  {\(C\_1\), \(D\_1\), \(\(-\[Gamma]\^2\) I\)}
                  }], "\[NegativeThinSpace]", ")"}]}], "  ", "<", "  ", 
          "0"}], 
        StyleBox["\n",
          FontSize->16]}]]]
}], "Text",
  FontFamily->"Courier",
  FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Proof.", "Section"],

Cell[TextData[{
  "It",
  StyleBox[" will ",
    FontFamily->"Courier"],
  "be show that statement III) is equivalent to statement II). The \
equivalence between I) and II)  can be found in [Doyle, J.C. et al, \
\"State-Space Solution to Standard ",
  Cell[BoxData[
      SubscriptBox[
        StyleBox["\[ScriptCapitalH]",
          FontSize->16], "2"]]],
  "and ",
  Cell[BoxData[
      SubscriptBox[
        StyleBox["\[ScriptCapitalH]",
          FontSize->16], "\[Infinity]"]]],
  "Control Problems\", IEEE Trans. Aut. Contr., AC-34 (1998), pp 831 - 847; \
Iwasaki,T. and R. Skelton\", All controllers for the General ",
  Cell[BoxData[
      SubscriptBox[
        StyleBox["\[ScriptCapitalH]",
          FontSize->16], "\[Infinity]"]]],
  " Control Problem: LMI Existence Conditions and State-Space Formulas\", \
Automatica,30 (1994), pp. 1307-1317."
}], "Text",
  TextAlignment->Left,
  TextJustification->0,
  FontFamily->"Courier",
  FontWeight->"Bold"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applying the LDU decomposition.", "Section"],

Cell["\<\

To proof the equivalence between statements II) and  III), the matrix in \
statement III) is factored by a LDU decomposition. The resulting diagonal \
matrix express the conditions in statement II).\
\>", "Text",
  CellMargins->{{12, Inherited}, {Inherited, Inherited}},
  FontFamily->"Courier",
  FontWeight->"Bold"],

Cell["\<\

Setting the matrices to be noncommutative.\
\>", "Text"],

Cell["\<\

SetNonCommutative[
\tA, B, C1, D1, X,tp[B], tp[C1], tp[D1]
\t];\
\>", "Input"],

Cell[TextData[{
  "Defining the LMI in statement III),  where  nu  = ",
  Cell[BoxData[
      \(\(\ \[Gamma]\^2\)\)]]
}], "Text"],

Cell["\<\

BigLMI = {{X**A+tp[A]**X, X**B, tp[C1]},
\t\t\t{Tp[B]**X, - Id, tp[D1]},
\t\t\t{C1, D1, -nu Id}};
tp[X]=X;
MatrixForm[BigLMI] \
\>", "Input"]
}, Open  ]],

Cell["\<\

Applying the LDU decomposition\
\>", "Text"],

Cell["\<\
{lower, diagon, upper, P} = NCLDUDecomposition[BigLMI
\t, Permutation->{{2,1,3},{1,3,2}}];\
\>", "Input"],

Cell["\<\

Let's examine the diagonal matrix. Those elements should be  less than  zero
\
\>", "Text"],

Cell["Table[Print[diagon[[i,i]] ], {i,3} ];", "Input"],

Cell[TextData[{
  "\nThe second line is just \n\t\t",
  Cell[BoxData[
      \(\(\ 
      \[Gamma]\^2\  - \ \(D\_1\) \(D\_1\)\&\(\ \ T\)\ \ \  > \ \ 0\)\)]],
  "\t\t\t\t\t\t\t\t\t1)\n\t\t\n\tand the third line is \n\t\n\t",
  Cell[BoxData[{
      \(\ \ \ \ X\ \ A\  + \ A\^T\ X\ \  + \ X\ B\ B\^T\ X\  +  ... \), 
      \(\t\t\t\t\t\ \ \ \ 
      \((X\ \ \ B\ \ \ \(D\_1\)\&\(\ \ T\)\  + \ \(C\_1\)\&\(\ \ T\)\ )\)\ 
          \((\ \ \[Gamma]\^2\  - \ \(D\_1\) \(D\_1\)\&\(\ \ T\))\)\^\(-1\)\ 
          \((X\ \ \ B\ \ \ \(D\_1\)\&\(\ \ T\)\  + \ \(C\_1\)\&\(\ \ T\)\ )
              \)\^T\  < \ 0\n\t\)}]],
  "     \t\t2)\nTherefore, statement III)  is equivalent to statement II). To \
prove the converse,\nIt suffices to  apply Schur complement in 2) considering \
that inequality 1) holds."
}], "Text"]
}, Open  ]]
},
FrontEndVersion->"4.0 for X",
ScreenRectangle->{{0, 1280}, {0, 1024}},
WindowToolbars->"RulerBar",
CellGrouping->Manual,
WindowSize->{736, 734},
WindowMargins->{{190, Automatic}, {118, Automatic}},
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PaperSize"->{612, 792},
"PaperOrientation"->"Portrait",
"Magnification"->1}
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1739, 51, 109, 6, 236, "Title"],

Cell[CellGroupData[{
Cell[1873, 61, 37, 0, 52, "Section"],
Cell[1913, 63, 4571, 91, 492, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6521, 159, 25, 0, 52, "Section"],
Cell[6549, 161, 962, 28, 99, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[7548, 194, 50, 0, 52, "Section"],
Cell[7601, 196, 328, 8, 82, "Text"],
Cell[7932, 206, 67, 3, 50, "Text"],
Cell[8002, 211, 89, 5, 72, "Input"],
Cell[8094, 218, 129, 4, 32, "Text"],
Cell[8226, 224, 152, 7, 102, "Input"]
}, Open  ]],
Cell[8393, 234, 55, 3, 50, "Text"],
Cell[8451, 239, 115, 3, 42, "Input"],
Cell[8569, 244, 102, 4, 68, "Text"],
Cell[8674, 250, 54, 0, 27, "Input"],
Cell[8731, 252, 805, 16, 231, "Text"]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

